#If DEBUG Then
'----------------------------------------------------------------------------------------
Imports System
Imports System.Threading
Imports System.Globalization
Imports Microsoft.VisualStudio.TestTools.UnitTesting 'Microsoft.VisualStudio.QualityTools.UnitTestFramework.dll (with "Copy Local = False")

<TestClass()> _
<Acknowledgment(Author:="Christoph Hafner", SourceUrl:="https://www.nuget.org/packages/SingleFile.VB.ExtString.FillIn/1.0.27", License:=GetType(MitLicense), Comment:="Leave this acknowledgment 'as-is' and you are legally licensed.")> _
Public Class Test_ExtString_FillIn

    'Private Fields
    Private _TestContext As TestContext

    'Public Properties

    Public Property TestContext() As TestContext
        Get
            Return _TestContext
        End Get
        Set(ByVal value As TestContext)
            _TestContext = value
        End Set
    End Property

    'Public Methods

    <TestMethod()> _
    <TestCategory("Extension Methods")> _
    Public Sub ExtString_FillIn()
        'Check whether the standard locale implementation for "de-CH", "fr-CH" and "it-CH" are correct
        CheckSwissLocales(TestContext)
        'Define format providers (and adjust them to ensure they are implemented correctly on your system)
        Dim myProvider1 As New CultureInfo("en-US", False)
        myProvider1.NumberFormat.NumberDecimalSeparator = "."
        myProvider1.NumberFormat.NumberGroupSeparator = ","
        myProvider1.NumberFormat.NumberGroupSizes = New Int32() {3}
        Dim myProvider2 As New CultureInfo("de-DE", False)
        myProvider2.NumberFormat.NumberDecimalSeparator = ","
        myProvider2.NumberFormat.NumberGroupSeparator = "."
        myProvider2.NumberFormat.NumberGroupSizes = New Int32() {3}
        Dim myProvider3 As New CultureInfo("de-CH", False)
        myProvider3.NumberFormat.NumberDecimalSeparator = "."
        myProvider3.NumberFormat.NumberGroupSeparator = "'"
        myProvider3.NumberFormat.NumberGroupSizes = New Int32() {3}
        'Initialize default format provider to de-CH, language to en-US
        Thread.CurrentThread.CurrentCulture = myProvider3
        Thread.CurrentThread.CurrentUICulture = CultureInfo.InvariantCulture
        'Test only exception that is expected
        Dim myFormat As String = Nothing
        Dim myMessage As String = Nothing
        Try
            myFormat.FillIn("a", "b", "c")
            Assert.Fail("A NullReferenceException should have been thrown!")
        Catch ex As Exception
            Assert.IsInstanceOfType(ex, GetType(NullReferenceException))
        End Try
        'No other exceptions should be thrown...
        Try
            'Test "{0:yyyy-MM-dd}: {3,10:0.00} { 4 : 0 } {5,10} {6:dd.MM.yyyy}"

            'Test a simple date format
            myFormat = "Today is {0:yyyy-MM-dd}."
            myMessage = myFormat.FillIn(New DateTime(2015, 12, 31))
            Assert.AreEqual("Today is 2015-12-31.", myMessage)
            'Test null value replacement
            myFormat = "The values are {0}, {1} and {2}."
            myMessage = myFormat.FillIn(Nothing, Nothing, 33)
            Assert.AreEqual("The values are [null], [null] and 33.", myMessage)
            'Test too many parameters
            myFormat = "Today is {1:yyyy-MM-dd}."
            myMessage = myFormat.FillIn(New DateTime(2015, 12, 31), New DateTime(2015, 11, 30), New DateTime(2015, 10, 31))
            Assert.AreEqual("Today is 2015-11-30.", myMessage)
            'Test too many placeholders
            myFormat = "The values are {0:0.###}, {1} and {2}."
            myMessage = myFormat.FillIn(18.45, True)
            Assert.AreEqual("The values are 18.45, True and {2}.", myMessage)
            'Test incorrect format
            myFormat = "Today is {0:yyyy-MM-dd}."
            myMessage = myFormat.FillIn(Guid.NewGuid())
            Assert.AreEqual("Today is {0:yyyy-MM-dd}.", myMessage)
            'Test confusing format
            myFormat = "public class {0} { //partial class, inherits {1}"
            myMessage = myFormat.FillIn("myClass", "myParentClass")
            Assert.AreEqual("public class myClass { //partial class, inherits myParentClass", myMessage)
            'Test format with leading spaces, test invert order
            myFormat = "{1,20}|{0,12:000,000.00}"
            myMessage = myFormat.FillIn(123.45, "Total amount USD")
            Assert.AreEqual("    Total amount USD|  000'123.45", myMessage)
            'Test multiple occurrences
            myFormat = "Raw: {0:0.########}, Rounded: {0:0}, Another number: {1}, Again the first one with 2 digits: {0:0.##}"
            myMessage = myFormat.FillIn(123.456789, 987.65)
            Assert.AreEqual("Raw: 123.456789, Rounded: 123, Another number: 987.65, Again the first one with 2 digits: 123.46", myMessage)
            myFormat = "Number: {0:N}"
            myMessage = myFormat.FillIn(myProvider1, 12345.6789)
            Assert.AreEqual("Number: 12,345.68", myMessage)
            'en-US
            myMessage = myFormat.FillIn(12345.6789, myProvider2)
            Assert.AreEqual("Number: 12.345,68", myMessage)
            'de-DE
            myMessage = myFormat.FillIn(myProvider3, 12345.6789, myProvider1)
            'First should be taken, second is a simple argument
            'de-CH
            Assert.AreEqual("Number: 12'345.68", myMessage)
        Catch ex As Exception When (Not TypeOf ex Is AssertFailedException) And (Not TypeOf ex Is AssertInconclusiveException) 'VB 10 support
            Assert.Fail("An exception of type " & ex.GetType().FullName & " occurred, but no exception should have been thrown!")
        End Try
    End Sub

    'Private Methods

    Private Sub CheckSwissLocales(context As TestContext)
        'Checks the 4 locales that were incorrectly installed in Windows 8.1 
        '(because of a bad decision of some Swiss government office to change the standard which was recalled later)
        CheckLocale("de-CH", 9999.99D, "9'999.99", context)
        CheckLocale("fr-CH", 9999.99D, "9'999.99", context)
        CheckLocale("it-CH", 9999.99D, "9'999.99", context)
        CheckLocale("rm-CH", 9999.99D, "9'999.99", context) 'interestingly enough this locale was never changed...
    End Sub

    Private Sub CheckLocale(locale As String, value As Decimal, expected As String, context As TestContext)
        Dim myFormatProvider As IFormatProvider = New CultureInfo(locale, False)
        Dim myString As String = value.ToString("N", myFormatProvider)
        If (myString <> expected) Then
            myFormatProvider = New CultureInfo(locale, True)
            Dim myString2 As String = value.ToString("N", myFormatProvider)
            If (myString2 = expected) Then
                'Locale was adjusted only as user override
                context.WriteLine("Warning: The system locale for ""{0}"" is incorrectly implemented on your system (correct format: ""{1}"", current format: ""{2}"") although the user override is correct (but that is not enough to avoid compatibility issues!). There was some confusion with several locales when Windows 8.1 was launched. Please download and use the Microsoft tool ""Locale Builder"" to correct this issue on the system level (it creates an MSI package that may be distributed company wide). Do not only adjust the user override in the regional settings of the control panel as applications that import or export data (for good reasons) ignore user overrides!", locale, expected, myString)
            Else
                'Locale is incorrect
                context.WriteLine("Warning: The system locale for ""{0}"" is incorrectly implemented on your system (correct format: ""{1}"", current format: ""{2}""). There was some confusion with several locales when Windows 8.1 was launched. Please download and use the Microsoft tool ""Locale Builder"" to correct this issue on the system level (it creates an MSI package that may be distributed company wide). Do not only adjust the user override in the regional settings of the control panel as applications that import or export data (for good reasons) ignore user overrides!", locale, expected, myString)
            End If
        End If
    End Sub

End Class
'----------------------------------------------------------------------------------------
#End If
